
#include "yahtzee.h"
#include "ai.h"

UInt32 delay;

void runComputerPlayer(ScoreSheetPtr score) {

	Int16 rerolls[5];
	Int16 i,j;
	Int16 chosen;
	FormPtr frm;
	ScoreSheetPtr s;
	
	s = (ScoreSheetPtr)MemHandleLock((MemHandle)score);
		

	
	for (j = 0; j < 2; j++) {
		for (i = 0; i < 5; i++) {
			rerolls[i]=0;
		}
		pickReRolls(getValues(), rerolls, (*s).score, scoredefinition);

		frm = FrmGetActiveForm();	
		
		InvertDice(rerolls);
		
		delay = SysTicksPerSecond();
		SysTaskDelay(delay);
		RollSelected();
		SysTaskDelay(delay);
		
	}
	
	chosen = placeScore(getValues(), (*s).score, scoredefinition);	
	MemHandleUnlock((MemHandle)score);
	
	handleScore(chosen);
	
}

Int16 placeScore (Int16 dice[], Score score[], ScoreDefinition scoredefinition[]) {

	Int16 best, possible, i;

	//Calculate the current score for each option
	for (i = 0; i < 13; i++) {
		if (score[i].value == -1) {
			score[i].result = scoredefinition[i].scorefunction(dice);	
		}
	}
	//If a box has done better than expected we should use it, the table is in
	//appoximate order of importance, ignore chance here
	best = -1;
	for (i = 0; i < 12; i++) {
		if ((score[i].result > scoredefinition[i].expected) && (score[i].value == -1)) {
			best = i;
		}
	}
	
	possible = -100;
	if (best == -1) {
		//Nothing matched, try for next worst match
		for (i = 12; i >= 0; i--) {
			if ((score[i].value == -1)) {
				if ((score[i].result - scoredefinition[i].expected) > possible) {
					best = i;
					possible = score[i].result - scoredefinition[i].expected;
				}
			}
		}
	}

	if (best ==-1) {
		//Nothing looks hopeful - let's bin it in the cheapest option
		for (i = 0; i < 13; i++) {
			if (score[i].value == -1) {
				best = i;
			}
		}
	}

	return best;
}

void pickReRolls(Int16 dice[], Int16 reroll[], Score score[], ScoreDefinition scoredefinition[]) {

	Int16 bestreroll[5];
	Int16 i,j;
	Int16 bestroll;

	//Now we have to pick a reroll scheme
	for (j = 0; j < 5; j++) {
		bestreroll[j] = 0;
	}
	
	bestroll = 5;
	for (i = 12; i >= 0; i--) {
		if (score[i].value == -1) {
			//Reset the rerolls
			for (j = 0; j < 5; j++) {
				reroll[j] = 1;
			}
			if (scoredefinition[i].rerollfunction(dice,reroll) < bestroll) {
				bestroll = scoredefinition[i].rerollfunction(dice,bestreroll);
			}
		}
	}

	for (i = 0; i < 5; i++) {
		reroll[i] = bestreroll[i];
	}

}

/** Functions to calculate rerolls */


Int16 rerollones(Int16 dice[],Int16 reroll[]) {
	return rerollsame(dice, reroll, 1);
}

Int16 rerolltwos(Int16 dice[],Int16 reroll[]) {
	return rerollsame(dice, reroll, 2);
}

Int16 rerollthrees(Int16 dice[], Int16 reroll[]) {
	return rerollsame(dice, reroll, 3);
}

Int16 rerollfours(Int16 dice[], Int16 reroll[]) {
	return rerollsame(dice, reroll, 4);
}

Int16 rerollfives(Int16 dice[], Int16 reroll[]) {
	return rerollsame(dice, reroll, 5);
}

Int16 rerollsixes(Int16 dice[], Int16 reroll[]) {
	return rerollsame(dice, reroll, 6);
}

Int16 rerollthreeofakind(Int16 dice[], Int16 reroll[]) {
	return rerollmost(dice,reroll);
}
Int16 rerollfourofakind(Int16 dice[], Int16 reroll[]){
	return rerollmost(dice,reroll);
}

//This isn't very good
Int16 rerollfullhouse(Int16 dice[], Int16 reroll[]){
	Int16 i;

	if (fullhouse(dice) != 0) {
		for (i = 0; i < 5; i++) {
			reroll[i] = 0;
		}
		return 0;
	}
	else {
		for (i = 0; i < 5; i++) {
			reroll[i] = 1;
		}
		return 5;
	}
}
//Always go for a long straight if possible
//This could be smarter
Int16 rerollshortstraight(Int16 dice[], Int16 reroll[]){

	return rerolllongstraight(dice, reroll);

}	

//We need to keep a 2,3,4,5 and reroll anything else.
Int16 rerolllongstraight(Int16 dice[], Int16 reroll[]){
	Int16 i,rolls;
	Int16 values[6];


	rolls = 0;
	if (longstraight(dice) != 0) {
		for (i = 0; i < 5; i++) {
			reroll[i] = 0;
		}
		return 0;
	}
	else {
		for (i = 0; i < 6; i++) {
			values[i] = 0;
		}
		
		for (i = 0; i < 5; i++) { 	
			values[dice[i]-1]++;
			if (values[dice[i]-1] > 1) {
				reroll[i] = 1;
				rolls++;
			}
			else {
				reroll[i] = 0;
			}
		}

		if ((values[0] > 0) && (values[5] > 0)) {
			for (i = 0; i < 5; i++) {
				if ((dice[i] == 1) || (dice[i] == 6)) {
						reroll[i] = 1;
						rolls++;
				}
			}
		}
	}
	return rolls;
}
Int16 rerollyahtzee(Int16 dice[], Int16 reroll[]){
	
	return rerollmost(dice,reroll);
}
Int16 rerollchance(Int16 dice[], Int16 reroll[]){
	Int16 i;
	for (i = 0; i < 5; i++) {
		reroll[i] = 1;
	}
	return 5;
}

/* Function to calculate the best reroll for getting all the same, 
 * prefers high rolls to low */
Int16 rerollmost(Int16 dice[], Int16 reroll[]) {
	Int16 best, number, count;
	Int16 i;
	Int16 values[6];
	
	for (i=0; i<6; i++) 
		values[i] =0;
	
	for (i = 0; i < 5; i++) 
		values[dice[i]-1]++;

	//Find the most
	best = 0;
	number = 0;
	for (i = 6; i >= 0; i--) {
		if (values[i] > best) {
			best = values[i];
			number = i + 1;
		}
	}
	
	//Set the rerolls
	count = 0;
	for (i = 0; i < 5; i++)  {
		if (dice[i] == number) {
			reroll[i] = 0;
		}
		else {
			reroll[i] = 1;
			count++;
		}
	}
	
	return count;
}
Int16 rerollsame(Int16 dice[], Int16 reroll[], Int16 number) {

	Int16 i;
	Int16 rolls = 0;

	for (i = 0; i < 5; i++) {
		if (dice[i] == number) {
			reroll[i] = 0;
		}
		else {
			reroll[i] = 1;
			rolls++;
		}
	}
	return rolls;
}


