/** (C) 2002 Peter Stevens
 * This is licensed under the GNU GPL
 *
 * See the file copying for details
 */


/* Main code for Dice Game */

#include "yahtzee.h"
#include "yahtzeeRsc.h"
#include "ai.h"
#include "ui.h"

UInt16 rolls = 0;
UInt16 current = 0;
UInt16 players = 0;
UInt16 finished = 0;
UInt16 playertype = 0;
ScoreSheetPtr scores[];

static Boolean ScoreTableHandleEvent (EventPtr event) {
	
	Boolean handled = false;
    FormPtr frm;
    
	CALLBACK_PROLOGUE
	
	finished = 0;
	switch (event->eType) {
    case frmOpenEvent:
		frm = FrmGetActiveForm();
		declareWinner();
		FrmDrawForm(frm);
		handled = true;
		break;
    
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		
		case NewGame :
			resetScores();
			current = 0;
			FrmGotoForm(MainForm);
		default :
		}

	default:
	
	}
	
	CALLBACK_EPILOGUE
	return handled;
    
}


static Boolean MainFormHandleEvent (EventPtr event) {
	
    Boolean handled = false;
    FormPtr frm;
   	int i, response;

    CALLBACK_PROLOGUE

    switch (event->eType) {
    case frmOpenEvent:
		frm = FrmGetActiveForm();
		DiceInit(frm, Dice1);
		DiceInit(frm, Dice2);
		DiceInit(frm, Dice3);
		DiceInit(frm, Dice4);
		DiceInit(frm, Dice5);
		FrmDrawForm(frm);

		runPlayer();
		handled = true;
		break;

	case menuEvent:
		MenuEraseStatus(NULL);

		switch(event->data.menu.itemID) {
		
		case MainOptionsSetPlayers:
			//Do stuff

			//Pop up an alert warning that this will quit the game
			response = FrmAlert(ChangePlayers);
			if (response == 0) {
				resetScores();
				for (i = 0; i < 5; i++) {
					DiceDestroy (FrmGetActiveForm(), Dice1 + i);
				}

				FrmGotoForm(NamePlayers);
			}
			break;

		case MainOptionsNewGame:
			//Do stuff

			//Pop up an alert warning that this will quit the game
			response = FrmAlert(ResetScores);
			if (response == 0) {
				resetScores();
				current = 0;
				rolls = 0;
				FrmGotoForm(MainForm);
			}
			break;

		case MainHelpUpper :
			FrmAlert(AlertHelpUpper);
			break;

		case MainHelpBonus :
			FrmAlert(AlertHelpBonus);
			break;

		case MainHelpLower3 :
			FrmAlert(AlertHelpLower3);
			break;

		case MainHelpFull:
			FrmAlert(AlertHelpFull);
			break;

		case MainHelpStraights :
			FrmAlert(AlertHelpStraights);
			break;
		
		case MainHelpYahtzee :
			FrmAlert(AlertHelpYahtzee);
			break;

		case MainHelpAbout:
			FrmAlert(AlertHelpAbout);
			break;
		
		default :
		
		}
		

		handled = true;
		break;

	case penDownEvent : 
		if (playertype == 1) {
			frm = FrmGetActiveForm();
				
			handled = handled || DiceTapped(frm, Dice1, event);
			handled = handled || DiceTapped(frm, Dice2, event);
			handled = handled || DiceTapped(frm, Dice3, event);
			handled = handled || DiceTapped(frm, Dice4, event);
			handled = handled || DiceTapped(frm, Dice5, event);
		
		}
	
	break;

    case ctlSelectEvent:
		if (playertype == 1) {
			switch(event->data.ctlSelect.controlID) {
	
				case MainRoll :
					if (rolls < 2) {
						frm = FrmGetActiveForm();
						DiceRoll(frm, Dice1);
						DiceRoll(frm, Dice2);
						DiceRoll(frm, Dice3);
						DiceRoll(frm, Dice4);
						DiceRoll(frm, Dice5);
						rolls = rolls + 1;		
					}
				break;

			default :

				if (((event->data.ctlSelect.controlID) >= ScoreBtn1) &&
					((event->data.ctlSelect.controlID) <= ScoreBtn13)) {
					handleScore(event->data.ctlSelect.controlID -ScoreBtn1);			
				}
			}
	
			break;

    	default:
        	break;
		}
    }
	
	updateNumber(RollsRemaining, 2 - rolls);	

	if (finished == 1) {
		FrmGotoForm(ScoreTable);
	}
	
    CALLBACK_EPILOGUE

    return handled;
}

		
static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Int16	formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case MainForm:
	    FrmSetEventHandler(frm, MainFormHandleEvent);
	    break;
	case ScoreTable:
	    FrmSetEventHandler(frm, ScoreTableHandleEvent);
	    break;
	case NamePlayers:
	    FrmSetEventHandler(frm, NamePlayersHandleEvent);
	    break;
	}
	handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
static Int16 StartApplication(void)
{
	
	FrmGotoForm(NamePlayers);

	
	return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
	FormPtr frm;
	int i;

	//We need to retrieve all the memory now
	for (i = 0; i < players; i++) {
		MemHandleFree((MemHandle)scores[i]);
	}
	
	frm = FrmGetFormPtr(MainForm);
	if (frm) {
		for (i = 0; i < 5; i++) {
			DiceDestroy(frm, Dice1 + i);
		}
	}
	FrmSaveAllForms();
    FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
    Int16 err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Int16 err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

	err = StartApplication();
	if (err) return err;

	EventLoop();
	StopApplication();

    } else {
	return sysErrParamErr;
    }

    return 0;
}


//This function inverts the selected dice
void InvertDice(UInt16 rerolls[]) {

	int i;
	FormPtr frm = FrmGetActiveForm();

	//Now highlight the dice we're changinging and wait for a second
	for (i = 0; i < 5; i++) {
		if (rerolls[i] == 1) {
			DiceInvert(frm, Dice1+i);
		}
	}
}

//This function rolls the selected dice.
void RollSelected() {

		FormPtr frm = FrmGetActiveForm();

		DiceRoll(frm, Dice1);
		DiceRoll(frm, Dice2);
		DiceRoll(frm, Dice3);
		DiceRoll(frm, Dice4);
		DiceRoll(frm, Dice5);

}


void runPlayer() {
	ScoreSheetPtr s;

	//Run the ai / event handler for the current player

	s = (ScoreSheetPtr)MemHandleLock((MemHandle)scores[current]);
	playertype = (*s).playertype;
	drawScoreBoard(s);
	DrawAll(Dice1);


	switch ((*s).playertype) {
		case 2 :
			MemHandleUnlock((MemHandle)scores[current]);
			runComputerPlayer(scores[current]);
			break;
		default:
			MemHandleUnlock((MemHandle)scores[current]);
			//current = nextPlayer(current);
			break;
	}
	
}

Boolean NamePlayersHandleEvent (EventPtr event) {
	Boolean handled = false;
	FormPtr frm;
	int i,j;
	UInt16 playertype;
	
	FieldPtr	fld;
	MemHandle	oldfieldhandle, newfieldhandle;
	char * h;
//	ListPtr lst;
	
	CALLBACK_PROLOGUE
	frm = FrmGetActiveForm();
	
	switch (event->eType) {
	case frmOpenEvent:
		
		//We need to fill the text boxes with the current player names
		for (i = 0; i < players; i++) {
			if (scores[i]) {
			ScoreSheetPtr s = (ScoreSheetPtr)MemHandleLock((MemHandle)scores[i]);
			fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, NamePlayer1 + i));
			oldfieldhandle = FldGetTextHandle(fld);
			
			
			newfieldhandle = MemHandleNew(13);
			h = (char *)MemHandleLock(newfieldhandle);
			
			StrCopy(h, (*s).name);
			MemHandleUnlock(newfieldhandle);
			
			FldSetTextHandle(fld,newfieldhandle);
			FldDrawField(fld);
				
			if (oldfieldhandle) {
				MemHandleFree(oldfieldhandle);
			}

			MemHandleUnlock((MemHandle)scores[i]);
			}
		}
	
		//Now we need to set the number of players in the list 
		//lst	= FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, NoPlayers));
		//LstSetSelection(lst, players-1);
		//LstDrawList(lst);
		//LstMakeItemVisible(lst, players-1);	
		frm = FrmGetActiveForm();
		FrmDrawForm(frm);
		handled = true;
		break;

	
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		
		case NamePlayers :
			//We need to set the names in the scoresheet here
			j = 0;
			for (i = 0; i < 6; i++) {
				
				fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, NamePlayer1 + i));
				oldfieldhandle = FldGetTextHandle(fld);
				
				//Now set their type
				playertype = FrmGetControlValue(frm, FrmGetObjectIndex(frm,PlayerType1+i)) + 1;
				
				//Only if we have an entry should we add a player to the game	
				if (oldfieldhandle) {
					h = (char *)MemHandleLock(oldfieldhandle);
					scores[j] = InitScoreSheet(h,playertype);
					j++;
					MemHandleUnlock(oldfieldhandle);
				}
			}
			
			players = j;
			if (j != 0) {	
				FrmGotoForm(MainForm);
			}
			break;

			default :
		}
	default :
		handled = false;
	}

	CALLBACK_EPILOGUE
	return handled;
}


/** This returns the number of the next player */

UInt16 nextPlayer(UInt16 current) {
	if (++current == players) {
		current = 0;
	}
	return current;

	
}

/** This function finds the winning player and displays the scores onscreen */
void declareWinner () {

	UInt16 i;
	UInt16 maxScore = 0;
	char winningPlayer[] = "NAMEOFPLAYER";
	UInt16 status = 0;
	ScoreSheetPtr s;
	
	for (i = 0; i < players; i++) {
		s = (ScoreSheetPtr)MemHandleLock((MemHandle)scores[i]);
		if ((*s).total > maxScore) {
			maxScore = (*s).total;
			StrCopy(winningPlayer, (*s).name);
			status = 0;
		}
		else if ((*s).total == maxScore) {
			status = 1;
		}
		updateLabel(Player1 + i, (*s).name);
		updateNumber(FinalScore1 + i, (*s).total);
		MemHandleUnlock((MemHandle)scores[i]);
	}

	for (i = players; i < 6; i++) {
		updateLabel(Player1 + i, "");
		updateLabel(FinalScore1 + i, "");

	}

	
	updateLabel(Winner, winningPlayer);
}

/** This function resets the scores for a new game */
void resetScores() {
           
	ScoreSheetPtr s;
	char name[] = "NAMEOFPLAYER";
	int playertype;
	int i;

	for (i = 0; i < players; i++) {

		s = (ScoreSheetPtr)MemHandleLock((MemHandle)scores[i]);
		StrCopy(name, (*s).name);
		playertype = (*s).playertype;
		
		MemHandleUnlock((MemHandle)scores[i]);
		MemHandleFree((MemHandle)scores[i]);
		scores[i] = InitScoreSheet(name, playertype);
	}
}

